//==============================================================================
//	
//	Copyright (c) 2002-
//	Authors:
//	* Dave Parker <david.parker@comlab.ox.ac.uk> (University of Oxford, formerly University of Birmingham)
//  * Hongyang Qu <hongyang.qu@comlab.ox.ac.uk> (University of Oxford)
//	
//------------------------------------------------------------------------------
//	
//	This file is part of PRISM.
//	
//	PRISM is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	PRISM is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with PRISM; if not, write to the Free Software Foundation,
//	Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//==============================================================================

// includes
#include "PrismMTBDD.h"
#include <math.h>
#include <util.h>
#include <cudd.h>
#include <dd.h>
#include "PrismMTBDDGlob.h"
#include "jnipointer.h"

//------------------------------------------------------------------------------

// PCTL until probability 0 precomputation
// (there exists = min) (nondeterministic/mdp) (mtbdd)
// (i.e. compute states where THERE EXISTS a probability 0)
// (NB: actually compute states FOR which ALL probabilities are >0 and then do a NOT)

JNIEXPORT jlong __jlongpointer JNICALL Java_mtbdd_PrismMTBDD_PM_1Prob0E
(
JNIEnv *env,
jclass cls,
jlong __jlongpointer t01, 	// 0-1 trans matrix
jlong __jlongpointer r,	// reachable states
jlong __jlongpointer ndm,	// nondeterminism mask
jlong __jlongpointer rv,	// row vars
jint num_rvars,
jlong __jlongpointer cv,	// col vars
jint num_cvars,
jlong __jlongpointer ndv,	// nondet vars
jint num_ndvars,
jlong __jlongpointer phi,	// phi(b1)
jlong __jlongpointer psi	// psi(b2)
)
{
  /*
	DdNode *trans01 = jlong_to_DdNode(t01);		// 0-1 trans matrix
	DdNode *reach = jlong_to_DdNode(r);		// reachable states
	DdNode *mask = jlong_to_DdNode(ndm);		// nondeterminism mask
	DdNode **rvars = jlong_to_DdNode_array(rv);	// row vars
	DdNode **cvars = jlong_to_DdNode_array(cv);	// col vars	
	DdNode **ndvars = jlong_to_DdNode_array(ndv);	// nondet vars
	DdNode *b1 = jlong_to_DdNode(phi);		// b1
	DdNode *b2 = jlong_to_DdNode(psi);		// b2
  */	
	
  DdNode *trans = Cudd_addBddThreshold(ddman, jlong_to_DdNode(t01), 1);		// 0-1 trans matrix
  Cudd_Ref(trans);
  DdNode *reach = Cudd_addBddThreshold(ddman, jlong_to_DdNode(r), 1);		// reachable states
  Cudd_Ref(reach);
	DdNode *mask = Cudd_addBddThreshold(ddman, jlong_to_DdNode(ndm), 1);		// nondeterminism mask
  Cudd_Ref(mask);
  DdNode **rvars_add = jlong_to_DdNode_array(rv);	// row vars
  DdNode **cvars_add = jlong_to_DdNode_array(cv);	// col vars
  DdNode **ndvars_add = jlong_to_DdNode_array(ndv);	// nondet vars
  DdNode *b1 = Cudd_addBddThreshold(ddman, jlong_to_DdNode(phi), 1);		// b1
  Cudd_Ref(b1);
  DdNode *b2 = Cudd_addBddThreshold(ddman, jlong_to_DdNode(psi), 1);		// b2
  Cudd_Ref(b2);
  DdNode *rvars[num_rvars];
  DdNode *cvars[num_cvars];
  DdNode *ndvars[num_ndvars];
 
  int i;
  for(i=0; i<num_rvars; i++)
    rvars[i] = Cudd_bddIthVar(ddman, rvars_add[i]->index);
  for(i=0; i<num_cvars; i++)
    cvars[i] = Cudd_bddIthVar(ddman, cvars_add[i]->index);
  for(i=0; i<num_ndvars; i++)
  ndvars[i] = Cudd_bddIthVar(ddman, ndvars_add[i]->index);

	DdNode *sol=NULL, *tmp, *tmp1, *tmp2;
	bool done;
	int iters;

	// timing stuff
	long start1, start2, start3, stop;
	double time_taken, time_for_setup, time_for_iters;

	// start clock
	start1 = util_cpu_time();
	
	// reachability fixpoint loop
	/*Cudd_Ref(b2);
	sol = b2;
	done = false;
	iters = 0;
	while (!done) {
	
		iters++;
		
		Cudd_Ref(sol);
		tmp = DD_PermuteVariables(ddman, sol, rvars, cvars, num_cvars);
		Cudd_Ref(trans01);
		tmp = DD_And(ddman, tmp, trans01);
		tmp = DD_ThereExists(ddman, tmp, cvars, num_cvars);
		
		Cudd_Ref(mask);
		tmp = DD_Or(ddman, tmp, mask);
		tmp = DD_ForAll(ddman, tmp, ndvars, num_ndvars);
		
		Cudd_Ref(b1);
		tmp = DD_And(ddman, b1, tmp);		
		Cudd_Ref(b2);
		tmp = DD_Or(ddman, b2, tmp);
		
		if (tmp == sol) {
			done = true;
		}
		Cudd_RecursiveDeref(ddman, sol);
		sol = tmp;
	}
	sol = DD_PermuteVariables(ddman, sol, cvars, rvars, num_cvars);
	
	// actual answer is states NOT reachable
	Cudd_Ref(reach);
	sol = DD_And(ddman, reach, DD_Not(ddman, sol));*/

	done = false;
	DdNode *cube = Cudd_bddComputeCube(ddman, cvars, NULL, num_cvars);
	Cudd_Ref(cube);
	DdNode *cube1 = Cudd_bddComputeCube(ddman, ndvars, NULL, num_ndvars);
	Cudd_Ref(cube1);
	DdNode *trans01 = Cudd_bddAnd(ddman, trans, b1); 
	Cudd_Ref(trans01);
	/*tmp = Cudd_bddAnd(ddman, mask, b1);
	Cudd_Ref(tmp);
	Cudd_RecursiveDeref(ddman, mask);
	mask = tmp;*/
	Cudd_Ref(b2);
	sol = b2;
	iters = 0;

	while(!done) {
	  iters++;
	  tmp1 = Cudd_bddSwapVariables(ddman, sol, rvars, cvars, num_rvars);
	  Cudd_Ref(tmp1);
	  tmp = Cudd_bddAndAbstract(ddman, tmp1, trans01, cube);
	  Cudd_Ref(tmp);
	  Cudd_RecursiveDeref(ddman, tmp1);

		tmp1 = Cudd_bddOr(ddman, tmp, mask);
		Cudd_Ref(tmp1);
		Cudd_RecursiveDeref(ddman, tmp);
		tmp = Cudd_bddUnivAbstract(ddman, tmp1, cube1);
		Cudd_Ref(tmp);
		Cudd_RecursiveDeref(ddman, tmp1);

		tmp2 = Cudd_bddAnd(ddman, tmp, b1);
		Cudd_Ref(tmp2);
		Cudd_RecursiveDeref(ddman, tmp);
		tmp = tmp2;

		tmp1 = Cudd_bddOr(ddman, tmp, b2);
		Cudd_Ref(tmp1);
		Cudd_RecursiveDeref(ddman, tmp);

		if(tmp1 == sol) {
			Cudd_RecursiveDeref(ddman, tmp1);
			break;
		} else {
			Cudd_RecursiveDeref(ddman, sol);
			sol = tmp1;
		}
	}

	// actual answer is states NOT reachable
	tmp = Cudd_bddAnd(ddman, reach, Cudd_Not(sol));
	Cudd_Ref(tmp);
	Cudd_RecursiveDeref(ddman, sol);
	tmp1 = Cudd_BddToAdd(ddman, tmp);
	Cudd_Ref(tmp1);
	Cudd_RecursiveDeref(ddman, tmp);
	sol = tmp1;

	Cudd_RecursiveDeref(ddman, cube1);
	Cudd_RecursiveDeref(ddman,cube);
	Cudd_RecursiveDeref(ddman,trans01);
	Cudd_RecursiveDeref(ddman,mask);
	Cudd_RecursiveDeref(ddman,reach);
	Cudd_RecursiveDeref(ddman,trans);
	Cudd_RecursiveDeref(ddman,b1);
	Cudd_RecursiveDeref(ddman,b2);

	// stop clock
	time_taken = (double)(util_cpu_time() - start1)/1000;
	time_for_setup = 0;
	time_for_iters = time_taken;

	// print iterations/timing info
	PM_PrintToMainLog(env, "\nProb0E: %d iterations in %.2f seconds (average %.6f, setup %.2f)\n", iters, time_taken, time_for_iters/iters, time_for_setup);

	return ptr_to_jlong(sol);
}

//------------------------------------------------------------------------------
