//==============================================================================
//	
//	Copyright (c) 2002-
//	Authors:
//	* Dave Parker <david.parker@comlab.ox.ac.uk> (University of Oxford, formerly University of Birmingham)
//	
//------------------------------------------------------------------------------
//	
//	This file is part of PRISM.
//	
//	PRISM is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	PRISM is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with PRISM; if not, write to the Free Software Foundation,
//	Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//==============================================================================

// includes
#include "PrismMTBDD.h"
#include <math.h>
#include <util.h>
#include <cudd.h>
#include <dd.h>
#include "PrismMTBDDGlob.h"
#include "jnipointer.h"

//------------------------------------------------------------------------------

// PCTL until probability 1 precomputation (probabilistic/dtmc) (mtbdd)

JNIEXPORT jlong __jlongpointer JNICALL Java_mtbdd_PrismMTBDD_PM_1Prob1
(
JNIEnv *env,
jclass cls,
jlong __jlongpointer t01, 	// 0-1 trans matrix
jlong __jlongpointer r,		// all reachable states
jlong __jlongpointer rv,	// row vars
jint num_rvars,
jlong __jlongpointer cv,	// col vars
jint num_cvars,
jlong __jlongpointer phi,	// phi(b1)
jlong __jlongpointer psi,	// psi(b2)
jlong __jlongpointer _no	// no
)
{
	/*
	DdNode *trans01 = jlong_to_DdNode(t01);		// 0-1 trans matrix
	DdNode *reach = jlong_to_DdNode(r);			// all reachable states
	DdNode *b1 = jlong_to_DdNode(phi);			// b1
	DdNode *b2 = jlong_to_DdNode(psi);			// b2
	DdNode *no = jlong_to_DdNode(_no);			// no
	DdNode **rvars = jlong_to_DdNode_array(rv);	// row vars
	DdNode **cvars = jlong_to_DdNode_array(cv);	// col vars
	*/

	DdNode *sol=NULL, *tmp, *tmp1;
	bool done;
	int iters;

  DdNode *trans = Cudd_addBddThreshold(ddman, jlong_to_DdNode(t01), 1);		// 0-1 trans matrix
  Cudd_Ref(trans);
  DdNode *reach = Cudd_addBddThreshold(ddman, jlong_to_DdNode(r), 1);		// reachable states
  Cudd_Ref(reach);
  DdNode **rvars_add = jlong_to_DdNode_array(rv);	// row vars
  DdNode **cvars_add = jlong_to_DdNode_array(cv);	// col vars
  DdNode *no = Cudd_addBddThreshold(ddman, jlong_to_DdNode(_no), 1);		// b1
  Cudd_Ref(no);
  DdNode *b1 = Cudd_addBddThreshold(ddman, jlong_to_DdNode(phi), 1);		// b2
  Cudd_Ref(b1);
  DdNode *b2 = Cudd_addBddThreshold(ddman, jlong_to_DdNode(psi), 1);		// b2
  Cudd_Ref(b2);
  DdNode *rvars[num_rvars];
  DdNode *cvars[num_cvars];
 
  int i;
  for(i=0; i<num_rvars; i++)
    rvars[i] = Cudd_bddIthVar(ddman, rvars_add[i]->index);
  for(i=0; i<num_cvars; i++)
    cvars[i] = Cudd_bddIthVar(ddman, cvars_add[i]->index);

	// timing stuff
	long start1, start2, start3, stop;
	double time_taken, time_for_setup, time_for_iters;

	// start clock
	start1 = util_cpu_time();
	
	// reachability fixpoint loop
	/*Cudd_Ref(no);
	sol = no;
	done = false;
	iters = 0;
	while (!done) {
	
		iters++;
		
		Cudd_Ref(sol);
		tmp = DD_PermuteVariables(ddman, sol, rvars, cvars, num_cvars);
		Cudd_Ref(trans01);
		tmp = DD_And(ddman, tmp, trans01);
		tmp = DD_ThereExists(ddman, tmp, cvars, num_cvars);
		
		Cudd_Ref(b1);
		tmp = DD_And(ddman, b1, tmp);		
		Cudd_Ref(b2);
		tmp = DD_And(ddman, DD_Not(ddman, b2), tmp);
		Cudd_Ref(no);
		tmp = DD_Or(ddman, no, tmp);
		
		if (tmp == sol) {
			done = true;
		}
		Cudd_RecursiveDeref(ddman, sol);
		sol = tmp;
	}
	sol = DD_PermuteVariables(ddman, sol, cvars, rvars, num_cvars);
	
	// actual answer is states NOT reachable
	Cudd_Ref(reach);
	sol = DD_And(ddman, reach, DD_Not(ddman, sol));*/

	DdNode *cube = Cudd_bddComputeCube(ddman, cvars, NULL, num_cvars);
	Cudd_Ref(cube);
	tmp = Cudd_bddOr(ddman, no, b2);
	Cudd_Ref(tmp);
	tmp1 = Cudd_bddAnd(ddman, reach, Cudd_Not(tmp));
	Cudd_Ref(tmp1);
	Cudd_RecursiveDeref(ddman, tmp);
	DdNode *trans01 = Cudd_bddAnd(ddman, trans, tmp1); 
	Cudd_Ref(trans01);
	Cudd_RecursiveDeref(ddman, tmp1);

	done = false;
	iters = 1;
	
	tmp = Cudd_bddSwapVariables(ddman, no, rvars, cvars, num_rvars);
	Cudd_Ref(tmp);
	tmp1 = Cudd_bddAndAbstract(ddman, tmp, trans01, cube);
	Cudd_Ref(tmp1);
	Cudd_RecursiveDeref(ddman, tmp);
	if(tmp1 == Cudd_ReadLogicZero(ddman)) {
	  done = true;
	  Cudd_RecursiveDeref(ddman, tmp1);
	}

	while(!done) {
	  iters++;
	  if(sol == NULL) {
	    sol = tmp1;
	    Cudd_Ref(tmp1);
	    tmp = tmp1;
	  } else {
	    tmp = Cudd_bddAnd(ddman, tmp1, Cudd_Not(sol));
	    Cudd_Ref(tmp);
	    Cudd_RecursiveDeref(ddman, tmp1);
	    if(tmp == Cudd_ReadLogicZero(ddman)) {
	      Cudd_RecursiveDeref(ddman, tmp);
	      break;
	    } else {
	      tmp1 = Cudd_bddOr(ddman, sol, tmp);
	      Cudd_Ref(tmp1);
	      Cudd_RecursiveDeref(ddman, sol);
	      sol = tmp1;
	    }
	  }
	  tmp1 = Cudd_bddSwapVariables(ddman, tmp, rvars, cvars, num_rvars);
	  Cudd_Ref(tmp1);
	  Cudd_RecursiveDeref(ddman, tmp);
	  tmp = Cudd_bddAndAbstract(ddman, tmp1, trans01, cube);
	  Cudd_Ref(tmp);
	  Cudd_RecursiveDeref(ddman, tmp1);
	  tmp1 = tmp;
	}

	if(sol != NULL) {
	  tmp = Cudd_bddOr(ddman, sol, no);
	  Cudd_Ref(tmp);
	  Cudd_RecursiveDeref(ddman, sol);
	  sol = tmp;
	} else {
	  Cudd_Ref(no);
	  sol = no;
	}

	
	// actual answer is states NOT reachable
	tmp = Cudd_bddAnd(ddman, reach, Cudd_Not(sol));
	Cudd_Ref(tmp);
	Cudd_RecursiveDeref(ddman, sol);
	tmp1 = Cudd_BddToAdd(ddman, tmp);
	Cudd_Ref(tmp1);
	Cudd_RecursiveDeref(ddman, tmp);
	sol = tmp1;

	Cudd_RecursiveDeref(ddman,cube);
	Cudd_RecursiveDeref(ddman,trans01);
	Cudd_RecursiveDeref(ddman,reach);
	Cudd_RecursiveDeref(ddman,trans);
	Cudd_RecursiveDeref(ddman,no);
	Cudd_RecursiveDeref(ddman,b2);
	Cudd_RecursiveDeref(ddman,b1);

	// stop clock
	time_taken = (double)(util_cpu_time() - start1)/1000;
	time_for_setup = 0;
	time_for_iters = time_taken;

	// print iterations/timing info
	PM_PrintToMainLog(env, "\nProb1: %d iterations in %.2f seconds (average %.6f, setup %.2f)\n", iters, time_taken, time_for_iters/iters, time_for_setup);

	return ptr_to_jlong(sol);
}

//------------------------------------------------------------------------------
