//==============================================================================
//	
//	Copyright (c) 2002-
//	Authors:
//	* Dave Parker <david.parker@comlab.ox.ac.uk> (University of Oxford, formerly University of Birmingham)
//	* Gethin Norman <gethin.norman@comlab.ox.ac.uk> (University of Oxford, formerly University of Birmingham)
//  * Hongyang Qu <hongyang.qu@comlab.ox.ac.uk> (University of Oxford)
//	
//------------------------------------------------------------------------------
//	
//	This file is part of PRISM.
//	
//	PRISM is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	PRISM is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with PRISM; if not, write to the Free Software Foundation,
//	Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//==============================================================================

// includes
#include "PrismMTBDD.h"
#include <math.h>
#include <util.h>
#include <cudd.h>
#include <dd.h>
#include "PrismMTBDDGlob.h"
#include "jnipointer.h"

//------------------------------------------------------------------------------

// PCTL until probability 1 precomputation
// (for all = min) (nondeterministic/mdp) (mtbdd)
// (i.e. compute states FOR which ALL probabilities are 1)

JNIEXPORT jlong __jlongpointer JNICALL Java_mtbdd_PrismMTBDD_PM_1Prob1A
(
JNIEnv *env,
jclass cls,
jlong __jlongpointer t01, 	// 0-1 trans matrix
jlong __jlongpointer r,	// reachable states
jlong __jlongpointer ndm,	// nondeterminism mask
jlong __jlongpointer rv,	// row vars
jint num_rvars,
jlong __jlongpointer cv,	// col vars
jint num_cvars,
jlong __jlongpointer ndv,	// nondet vars
jint num_ndvars,
jlong __jlongpointer n,	// no
jlong __jlongpointer psi	// psi(b2)
)
{
  /*
	DdNode *trans01 = jlong_to_DdNode(t01);		// 0-1 trans matrix
	DdNode *reach = jlong_to_DdNode(r);		// reachable states
	DdNode *mask = jlong_to_DdNode(ndm);		// nondeterminism mask
	DdNode *no = jlong_to_DdNode(n);		// no
	DdNode *b2 = jlong_to_DdNode(psi);		// b2
	DdNode **rvars = jlong_to_DdNode_array(rv);	// row vars
	DdNode **cvars = jlong_to_DdNode_array(cv);	// col vars
	DdNode **ndvars = jlong_to_DdNode_array(ndv);	// nondet vars
  */	
	
  DdNode *trans = Cudd_addBddThreshold(ddman, jlong_to_DdNode(t01), 1);		// 0-1 trans matrix
  Cudd_Ref(trans);
  DdNode *reach = Cudd_addBddThreshold(ddman, jlong_to_DdNode(r), 1);		// reachable states
  Cudd_Ref(reach);
	DdNode *mask = Cudd_addBddThreshold(ddman, jlong_to_DdNode(ndm), 1);		// nondeterminism mask
  Cudd_Ref(mask);
  DdNode **rvars_add = jlong_to_DdNode_array(rv);	// row vars
  DdNode **cvars_add = jlong_to_DdNode_array(cv);	// col vars
  DdNode **ndvars_add = jlong_to_DdNode_array(ndv);	// nondet vars
  DdNode *no = Cudd_addBddThreshold(ddman, jlong_to_DdNode(n), 1);		// b1
  Cudd_Ref(no);
  DdNode *b2 = Cudd_addBddThreshold(ddman, jlong_to_DdNode(psi), 1);		// b2
  Cudd_Ref(b2);
  DdNode *rvars[num_rvars];
  DdNode *cvars[num_cvars];
  DdNode *ndvars[num_ndvars];
 
  int i;
  for(i=0; i<num_rvars; i++)
    rvars[i] = Cudd_bddIthVar(ddman, rvars_add[i]->index);
  for(i=0; i<num_cvars; i++)
    cvars[i] = Cudd_bddIthVar(ddman, cvars_add[i]->index);
  for(i=0; i<num_ndvars; i++)
  ndvars[i] = Cudd_bddIthVar(ddman, ndvars_add[i]->index);

	DdNode  *sol=NULL, *tmp, *tmp1;
	bool done;
	int iters;
	
	// timing stuff
	long start, stop;
	double time_taken, time_for_setup, time_for_iters;
	
	// start clock
	start = util_cpu_time();
	
	DdNode *cube = Cudd_bddComputeCube(ddman, cvars, NULL, num_cvars);
	Cudd_Ref(cube);
	DdNode *cube1 = Cudd_bddComputeCube(ddman, ndvars, NULL, num_ndvars);
	Cudd_Ref(cube1);
	tmp = Cudd_bddOr(ddman, no, b2);
	Cudd_Ref(tmp);
	tmp1 = Cudd_bddAnd(ddman, reach, Cudd_Not(tmp));
	Cudd_Ref(tmp1);
	Cudd_RecursiveDeref(ddman, tmp);
	DdNode *trans01 = Cudd_bddAndAbstract(ddman, trans, tmp1, cube1); 
	Cudd_Ref(trans01);
	Cudd_RecursiveDeref(ddman, tmp1);

	done = false;
	iters = 1;
	
	/*while (!done) {
		
		iters++;
		
		Cudd_Ref(sol);
		tmp = DD_SwapVariables(ddman, sol, rvars, cvars, num_rvars);
		Cudd_Ref(trans01);
		tmp = DD_ForAll(ddman, DD_Implies(ddman, trans01, tmp), cvars, num_cvars);
		
		Cudd_Ref(mask);
		tmp = DD_Or(ddman, tmp, mask);
		tmp = DD_ForAll(ddman, tmp, ndvars, num_ndvars);
		
		Cudd_Ref(notno);
		tmp = DD_And(ddman, notno, tmp);		
		Cudd_Ref(b2);
		tmp = DD_Or(ddman, b2, tmp);
		
		if (tmp == sol) {
			done = true;
		}
		Cudd_RecursiveDeref(ddman, sol);
		sol = tmp;
		}*/
	
	// first iteration
	tmp = Cudd_bddSwapVariables(ddman, no, rvars, cvars, num_rvars);
	Cudd_Ref(tmp);
	tmp1 = Cudd_bddAndAbstract(ddman, tmp, trans01, cube);
	Cudd_Ref(tmp1);
	Cudd_RecursiveDeref(ddman, tmp);
	if(tmp1 == Cudd_ReadLogicZero(ddman)) {
	  done = true;
	  Cudd_RecursiveDeref(ddman, tmp1);
	}

	while(!done) {
	  iters++;
	  if(sol == NULL) {
	    sol = tmp1;
	    Cudd_Ref(tmp1);
	    tmp = tmp1;
	  } else {
	    tmp = Cudd_bddAnd(ddman, tmp1, Cudd_Not(sol));
	    Cudd_Ref(tmp);
	    Cudd_RecursiveDeref(ddman, tmp1);
	    if(tmp == Cudd_ReadLogicZero(ddman)) {
	      Cudd_RecursiveDeref(ddman, tmp);
	      break;
	    } else {
	      tmp1 = Cudd_bddOr(ddman, sol, tmp);
	      Cudd_Ref(tmp1);
	      Cudd_RecursiveDeref(ddman, sol);
	      sol = tmp1;
	    }
	  }
	  tmp1 = Cudd_bddSwapVariables(ddman, tmp, rvars, cvars, num_rvars);
	  Cudd_Ref(tmp1);
	  Cudd_RecursiveDeref(ddman, tmp);
	  tmp = Cudd_bddAndAbstract(ddman, tmp1, trans01, cube);
	  Cudd_Ref(tmp);
	  Cudd_RecursiveDeref(ddman, tmp1);
	  tmp1 = tmp;
	}

	if(sol != NULL) {
	  tmp = Cudd_bddOr(ddman, sol, no);
	  Cudd_Ref(tmp);
	  Cudd_RecursiveDeref(ddman, sol);
	  sol = tmp;
	} else {
	  Cudd_Ref(no);
	  sol = no;
	}

	
	// actual answer is states NOT reachable
	tmp = Cudd_bddAnd(ddman, reach, Cudd_Not(sol));
	Cudd_Ref(tmp);
	Cudd_RecursiveDeref(ddman, sol);
	tmp1 = Cudd_BddToAdd(ddman, tmp);
	Cudd_Ref(tmp1);
	Cudd_RecursiveDeref(ddman, tmp);
	sol = tmp1;

	Cudd_RecursiveDeref(ddman, cube1);
	Cudd_RecursiveDeref(ddman,cube);
	Cudd_RecursiveDeref(ddman,trans01);
	Cudd_RecursiveDeref(ddman,mask);
	Cudd_RecursiveDeref(ddman,reach);
	Cudd_RecursiveDeref(ddman,trans);
	Cudd_RecursiveDeref(ddman,no);
	Cudd_RecursiveDeref(ddman,b2);

	// stop clock
	time_taken = (double)(util_cpu_time() - start)/1000;
	time_for_setup = 0;
	time_for_iters = time_taken;
	
	// print iterations/timing info
	PM_PrintToMainLog(env, "\nProb1A: %d iterations in %.2f seconds (average %.6f, setup %.2f)\n", iters, time_taken, time_for_iters/iters, time_for_setup);
	
	return ptr_to_jlong(sol);
}

//------------------------------------------------------------------------------
