//==============================================================================
//	
//	Copyright (c) 2002-
//	Authors:
//	* Dave Parker <david.parker@comlab.ox.ac.uk> (University of Oxford, formerly University of Birmingham)
//  * Hongyang Qu <hongyang.qu@comlab.ox.ac.uk> (University of Oxford)
//	
//------------------------------------------------------------------------------
//	
//	This file is part of PRISM.
//	
//	PRISM is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	PRISM is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with PRISM; if not, write to the Free Software Foundation,
//	Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//==============================================================================

// includes
#include "PrismMTBDD.h"
#include <math.h>
#include <util.h>
#include <cudd.h>
#include <dd.h>
#include "PrismMTBDDGlob.h"
#include "jnipointer.h"

//------------------------------------------------------------------------------

// PCTL until probability 1 precomputation
// (there exists = max) (nondeterministic/mdp) (mtbdd)
// (i.e. compute states where THERE EXISTS a probability 1)

JNIEXPORT jlong __jlongpointer JNICALL Java_mtbdd_PrismMTBDD_PM_1Prob1E
(
JNIEnv *env,
jclass cls,
jlong __jlongpointer t01, 	// 0-1 trans matrix
jlong __jlongpointer r,	// reachable states
jlong __jlongpointer rv,	// row vars
jint num_rvars,
jlong __jlongpointer cv,	// col vars
jint num_cvars,
jlong __jlongpointer ndv,	// nondet vars
jint num_ndvars,
jlong __jlongpointer phi,	// phi(b1)
jlong __jlongpointer psi,	// psi(b2)
jlong __jlongpointer _no	// no
)
{
  /*
	DdNode *trans01 = jlong_to_DdNode(t01);		// 0-1 trans matrix
	DdNode *reach = jlong_to_DdNode(r);		// reachable states
	DdNode **rvars = jlong_to_DdNode_array(rv);	// row vars
	DdNode **cvars = jlong_to_DdNode_array(cv);	// col vars
	DdNode **ndvars = jlong_to_DdNode_array(ndv);	// nondet vars
	DdNode *b1 = jlong_to_DdNode(phi);		// b1
	DdNode *b2 = jlong_to_DdNode(psi);		// b2
	DdNode *no = jlong_to_DdNode(_no);			// no
  */

  DdNode *trans01 = Cudd_addBddThreshold(ddman, jlong_to_DdNode(t01), 1);		// 0-1 trans matrix
  Cudd_Ref(trans01);
  DdNode *reach = Cudd_addBddThreshold(ddman, jlong_to_DdNode(r), 1);		// reachable states
  Cudd_Ref(reach);
  DdNode **rvars_add = jlong_to_DdNode_array(rv);	// row vars
  DdNode **cvars_add = jlong_to_DdNode_array(cv);	// col vars
  DdNode **ndvars_add = jlong_to_DdNode_array(ndv);	// nondet vars
  DdNode *b1 = Cudd_addBddThreshold(ddman, jlong_to_DdNode(phi), 1);		// b1
  Cudd_Ref(b1);
  DdNode *b2 = Cudd_addBddThreshold(ddman, jlong_to_DdNode(psi), 1);		// b2
  Cudd_Ref(b2);
  DdNode *no = Cudd_addBddThreshold(ddman, jlong_to_DdNode(_no), 1);		// b2
  Cudd_Ref(no);
  DdNode *rvars[num_rvars];
  DdNode *cvars[num_cvars];
  DdNode *ndvars[num_ndvars];

  int i;
  for(i=0; i<num_rvars; i++)
    rvars[i] = Cudd_bddIthVar(ddman, rvars_add[i]->index);
  for(i=0; i<num_cvars; i++)
    cvars[i] = Cudd_bddIthVar(ddman, cvars_add[i]->index);
  for(i=0; i<num_ndvars; i++)
  ndvars[i] = Cudd_bddIthVar(ddman, ndvars_add[i]->index);


	DdNode *u, *v, *tmp, *tmp1, *tmp2, *tmp3;
	bool u_done, v_done;
	int iters;

	// timing stuff
	long start1, start2, start3, stop;
	double time_taken, time_for_setup, time_for_iters;

	// start clock
	start1 = util_cpu_time();
	
	// greatest fixed point so should start from true,
	// but for efficiency we use the passed in "no", which will
	// be the result of the first (outer) iteration
	/*Cudd_Ref(reach);
	Cudd_Ref(no);
	u = DD_And(ddman, reach, DD_Not(ddman, no));
	//u = DD_Constant(ddman, 1);
	u_done = false;
	iters = 0;
	
	while (!u_done) {
	
		// least fixed point - start from false
		v = DD_Constant(ddman, 0);
		v_done = false;
		
		while (!v_done) {
				
			iters++;
				
			Cudd_Ref(u);
			tmp = DD_SwapVariables(ddman, u, rvars, cvars, num_rvars);
			Cudd_Ref(trans01);
			tmp = DD_ForAll(ddman, DD_Implies(ddman, trans01, tmp), cvars, num_cvars);

			Cudd_Ref(v);
			tmp2 = DD_SwapVariables(ddman, v, rvars, cvars, num_rvars);
			Cudd_Ref(trans01);
			tmp2 = DD_ThereExists(ddman, DD_And(ddman, tmp2, trans01), cvars, num_cvars);

			tmp = DD_And(ddman, tmp, tmp2);
			tmp = DD_ThereExists(ddman, tmp, ndvars, num_ndvars);

			Cudd_Ref(b1);
			tmp = DD_And(ddman, b1, tmp);		
			Cudd_Ref(b2);
			tmp = DD_Or(ddman, b2, tmp);
			
			if (tmp == v) {
				v_done = true;
			}
			Cudd_RecursiveDeref(ddman, v);
			v = tmp;
		}
		if (v == u) {
			u_done = true;
		}
		Cudd_RecursiveDeref(ddman, u);
		u = v;
		}*/

	u_done = false;
	DdNode *cube = Cudd_bddComputeCube(ddman, cvars, NULL, num_cvars);
	Cudd_Ref(cube);
	DdNode *cube1 = Cudd_bddComputeCube(ddman, ndvars, NULL, num_ndvars);
	Cudd_Ref(cube1);

	// restrict trans01 to b1
	tmp = Cudd_bddAnd(ddman, trans01, b1);
	Cudd_Ref(tmp);
	Cudd_RecursiveDeref(ddman, trans01);
	trans01 = tmp;	

	DdNode *trans02 = Cudd_Not(trans01); 
	Cudd_Ref(trans02);
	//Cudd_RecursiveDeref(ddman, cube1);
	iters = 0;

	u = Cudd_bddAnd(ddman, reach, Cudd_Not(no));
	Cudd_Ref(u);
	while (!u_done) {
	  // least fixed point - start from false
	  //v = Cudd_ReadLogicZero(ddman);
	  v = b2;
	  Cudd_Ref(v);
	  v_done = false;
	  
	  tmp = Cudd_bddSwapVariables(ddman, u, rvars, cvars, num_rvars);
	  Cudd_Ref(tmp);
	  tmp1 = Cudd_bddOr(ddman, trans02, tmp);
	  Cudd_Ref(tmp1);
	  Cudd_RecursiveDeref(ddman, tmp);
	  tmp = Cudd_bddUnivAbstract(ddman, tmp1, cube);
	  Cudd_Ref(tmp);
	  Cudd_RecursiveDeref(ddman, tmp1);

	  tmp1 = v;
	  Cudd_Ref(tmp1);
	  while (!v_done) {
	    iters++;

	    //tmp2 = Cudd_bddSwapVariables(ddman, v, rvars, cvars, num_rvars);
	    tmp2 = Cudd_bddSwapVariables(ddman, tmp1, rvars, cvars, num_rvars);
	    Cudd_Ref(tmp2);
	    Cudd_RecursiveDeref(ddman, tmp1);
	    tmp1 = Cudd_bddAndAbstract(ddman, tmp2, trans01, cube);
	    Cudd_Ref(tmp1);
	    Cudd_RecursiveDeref(ddman, tmp2);

	    tmp2 = Cudd_bddAndAbstract(ddman, tmp, tmp1, cube1);
	    Cudd_Ref(tmp2);
	    //Cudd_RecursiveDeref(ddman, tmp);
	    Cudd_RecursiveDeref(ddman, tmp1);

	    /*tmp1 = Cudd_bddAnd(ddman, b1, tmp2);
	    Cudd_Ref(tmp1);
	    Cudd_RecursiveDeref(ddman, tmp2);
	    tmp2 = Cudd_bddOr(ddman, b2, tmp1);
	    Cudd_Ref(tmp2);
	    Cudd_RecursiveDeref(ddman, tmp1);*/
	    /*tmp1 = Cudd_bddOr(ddman, b2, tmp2);
	    Cudd_Ref(tmp1);
	    Cudd_RecursiveDeref(ddman, tmp2);*/
	    tmp1 = Cudd_bddAnd(ddman, tmp2, Cudd_Not(v));
	    Cudd_Ref(tmp1);
	    Cudd_RecursiveDeref(ddman, tmp2);

	    if (tmp1 == Cudd_ReadLogicZero(ddman)/*v*/) {
	      v_done = true;
	      Cudd_RecursiveDeref(ddman, tmp1);
	    } else {
	      tmp2 = Cudd_bddOr(ddman, v, tmp1);
	      Cudd_Ref(tmp2);
	      Cudd_RecursiveDeref(ddman, v);
	      v = tmp2;
	    }
	    //Cudd_RecursiveDeref(ddman, v);
	    //v = tmp1;
	  }
	  Cudd_RecursiveDeref(ddman, tmp);
	  if (v == u) {
	    u_done = true;
	  }
	  Cudd_RecursiveDeref(ddman, u);
	  u = v;
	}

	tmp = Cudd_BddToAdd(ddman, u);
	Cudd_Ref(tmp);
	Cudd_RecursiveDeref(ddman, u);
	u = tmp;


	Cudd_RecursiveDeref(ddman,cube1);
	Cudd_RecursiveDeref(ddman,cube);
	Cudd_RecursiveDeref(ddman,trans02);
	Cudd_RecursiveDeref(ddman,reach);
	Cudd_RecursiveDeref(ddman,trans01);
	Cudd_RecursiveDeref(ddman,b1);
	Cudd_RecursiveDeref(ddman,b2);
	Cudd_RecursiveDeref(ddman,no);
	
	
	// stop clock
	time_taken = (double)(util_cpu_time() - start1)/1000;
	time_for_setup = 0;
	time_for_iters = time_taken;

	// print iterations/timing info
	PM_PrintToMainLog(env, "\nProb1E: %d iterations in %.2f seconds (average %.6f, setup %.2f)\n", iters, time_taken, time_for_iters/iters, time_for_setup);

	return ptr_to_jlong(u);
}

//------------------------------------------------------------------------------
