//==============================================================================
//  
//  Copyright (c) 2002-
//  Authors:
//  * Dave Parker <david.parker@comlab.ox.ac.uk> (University of Oxford, formerly University of Birmingham)
//  * Hongyang Qu <Hongyang.Qu@comlab.ox.ac.uk> (University of Oxford)
//  
//------------------------------------------------------------------------------
//  
//  This file is part of PRISM.
//  
//  PRISM is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  PRISM is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with PRISM; if not, write to the Free Software Foundation,
//  Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  
//==============================================================================

// includes
#include "PrismMTBDD.h"
#include <math.h>
#include <vector>
#include <map>
#include <util.h>
#include <cudd.h>
#include <dd.h>
#include <dv.h>
#include <odd.h>
#include "sparse.h"
#include "PrismMTBDDGlob.h"
#include "jnipointer.h"
#include <new>
#include "lp_lib.h"

#define LPBOUND 2
//------------------------------------------------------------------------------
// Tarjan's algorithm
using namespace std;

struct result 
{
  int index;
  int w0;
};

DdNode *mask;
DdNode *trans, *trans03, *trans04, *trans02, *allsccs;
DdNode *f;
DdNode *bddyes;
DdNode *cube; // for rvars
DdNode *cube1; // for ndvars
DdNode *cube2; // for cvars
DdNode *addcube1; //, *addcube2;
DdNode **rvars, **cvars, **ndvars; // they are ADDs
DdNode *sol, *probone;
DdNode **bddrvars;
DdNode **bddcvars;
DdNode **bddndvars;
DdNode *nontrivialsccs;
vector<DdNode *> *S;
map<DdNode *, int> *S2;
int num_rvars, num_cvars, num_ndvars;
int dividend;
bool Pmin;

int iters = 0;
int num_sccs = 0;
long count1 = 0;
long count2 = 0;
long count3 = 0;
long count4 = 0;
long count5 = 0;
long count6 = 0;  
long count7 = 0;  

void processSCC_LP(DdNode *scc, int nscc){

  // mtbdds
  DdNode *a = NULL, *tmp = NULL;
  // model stats
  int n;
  // sparse matrix
  NDSparseMatrix *ndsm = NULL;
  // timing stuff
  long start1, start2, stop;
  double time_taken, time_for_setup, time_for_lp;
  // lp stuff
  double *arr_reals = NULL;
  int *arr_ints = NULL;
  int num_lp_vars;
  double *lp_soln;
  double lp_result = 0.0, d1, d2;
  // misc
  int i, j, k, l1, h1, l2, h2, count;

  DdNode *addscc;
  DdNode *tmp3, *tmp4;
  DdNode *tmpsol, *sccsucc, *scctrans01;
  int *map_var;
  DdNode **states = NULL;
  ODDNode *odd;
  // lp stuff
  lprec *lp = NULL;
  // exception handling around whole function

  try {

    // start clocks 
    start1 = start2 = util_cpu_time();
  
    //printf("  creating BDDs...\n");
    addscc = Cudd_BddToAdd(ddman, scc);
    Cudd_Ref(addscc);
    
    scctrans01 = Cudd_bddAndAbstract(ddman, trans03, scc, cube); 
    Cudd_Ref(scctrans01);
    tmp4 = Cudd_bddSwapVariables(ddman, scctrans01, rvars, cvars, num_rvars);
    Cudd_Ref(tmp4);
    Cudd_RecursiveDeref(ddman, scctrans01);
    sccsucc = Cudd_bddExistAbstract(ddman, tmp4, cube1);
    Cudd_Ref(sccsucc);     
    Cudd_RecursiveDeref(ddman, tmp4);
    
    //printf(": performing value iteration...\n "); fflush(stdout);
    a = Cudd_addApply(ddman, Cudd_addTimes, trans, addscc);
    Cudd_Ref(a);
    Cudd_RecursiveDeref(ddman, addscc);
    tmp3 = Cudd_BddToAdd(ddman, sccsucc);
    Cudd_Ref(tmp3);   
    Cudd_RecursiveDeref(ddman, sccsucc);
    tmpsol = Cudd_addApply(ddman, Cudd_addTimes, sol, tmp3);
    Cudd_Ref(tmpsol);
    
    //printf("  generating ODD...\n");
    odd = build_odd(ddman, tmp3, rvars, num_rvars);
    n = odd->eoff + odd->toff;

    // Build sparse matrix
    ndsm = build_nd_sparse_matrix(ddman, a, rvars, cvars, num_rvars, ndvars, num_ndvars, odd);
    // Get number of transitions/choices
    
    
    // Store local copies of sparse matrix stuff
    double *non_zeros = ndsm->non_zeros;
    unsigned char *row_counts = ndsm->row_counts;
    int *row_starts = (int *)ndsm->row_counts;
    unsigned char *choice_counts = ndsm->choice_counts;
    int *choice_starts = (int *)ndsm->choice_counts;
    bool use_counts = ndsm->use_counts;
    unsigned int *cols = ndsm->cols;

    //printf("  allocating memory...\n");
    arr_ints = new int[n];
    arr_reals = new double[n];
    lp_soln = new double[n];
    states = new DdNode*[n];
    map_var = new int[n];

    //printf("  computing map_vars...\n");
    count = 0; 
    // Traverse sparse matrix to get info
    h1 = h2 = 0;
    for (i = 0; i < n; i++) {
      states[i] = single_index_to_bdd(ddman, i, rvars, num_rvars, odd);
      if (!use_counts) {
        l1 = row_starts[i];
        h1 = row_starts[i+1];
      } else {
        l1 = h1;
        h1 += row_counts[i];
      }
      if(l1==h1 /*|| lp_soln[i] == 1.0*/) {
        map_var[i] = -1;
        tmp4 = Cudd_addApply(ddman, Cudd_addTimes, tmpsol, states[i]); 
        Cudd_Ref(tmp4);
        lp_soln[i] = tmp4 == Cudd_ReadZero(ddman) ? 0.0 : DD_FindMax(ddman, tmp4);
        Cudd_RecursiveDeref(ddman, tmp4);
        Cudd_RecursiveDeref(ddman, states[i]);
        states[i] = NULL;
      } else {
        map_var[i] = count++;
        // assume we always perform precomputation so that
        // scc does not contain "yes" states.
        lp_soln[i] = 0.0;
      }
    }
    num_lp_vars = count;

    if ((lp=make_lp(0, num_lp_vars)) == NULL) throw "Could not create LP problem";
    // Lower verbosity: Warnings and errors only
    set_verbose(lp, IMPORTANT);
    // Set mode: will create row by row
    set_add_rowmode(lp, true);

    // Add constraints to LP problem
    //printf("  generating LP program...\n");
    int x;
    h1 = h2 = 0;
    for (x = 0; x < n; x++) { // h2: row index
      if (!use_counts) {
        l1 = row_starts[x];
        h1 = row_starts[x+1];
      } else {
        l1 = h1;
        h1 += row_counts[x];
      }
      for (j = l1; j < h1; j++) { // j: choice index; x+j: variable index 
        if (!use_counts) {
          l2 = choice_starts[j];
          h2 = choice_starts[j+1];
        } else {
          l2 = h2;
          h2 += choice_counts[j];
        }
        d1 = 0.0;
        d2 = 1.0;
        count = 1;
        for(k=l2; k<h2; k++) {
          if(map_var[cols[k]] == -1)
            d1 += non_zeros[k] * lp_soln[cols[k]];
          else {
            if(cols[k] != x) {
              arr_ints[count] = map_var[cols[k]]+1;
              arr_reals[count++] = -non_zeros[k];
            } else
              d2 -= non_zeros[k];
          }
        }
        arr_ints[0] = map_var[x]+1;
        arr_reals[0] = d2;

        if(Pmin)
          add_constraintex(lp, count, arr_reals, arr_ints, LE, d1);
        else
          add_constraintex(lp, count, arr_reals, arr_ints, GE, d1);
      }
    }

    // Finished building LP problem
    set_add_rowmode(lp, false);

    // Set objective function for LP
    x = 0;
    for(i=0; i<n; i++) 
      if(map_var[i] >= 0) {
        arr_ints[x] = map_var[i]+1;
        arr_reals[x++] = 1.0;
      }


    if(Pmin)
      set_maxim(lp);
		else
			set_minim(lp);
    set_obj_fnex(lp, x, arr_reals, arr_ints);
    delete[] lp_soln;
		lp_soln = NULL;
    

    // Get setup time
    stop = util_cpu_time();
    time_for_setup = (double)(stop - start2)/1000;
    start2 = stop;
    
    // Solve the LP, extract result
    //write_LP(lp, stdout);
		int res = solve(lp);
    if (res == 0) {     
      get_ptr_variables(lp, &lp_soln);
			/*count = 0;
			for (i=0; i<n; i++) {
        if(map_var[i] >= 0) {
          printf("X%d = %g    ", map_var[i], lp_soln[map_var[i]]);
          count++;
        }
        if(count == 8) {
          printf("\n");
          count = 0;
        }
      }
      if(count)
			printf("\n");*/

      for(i=0; i<n; i++) 
        if(map_var[i] >= 0) {
          tmp4 = Cudd_addConst(ddman, lp_soln[map_var[i]]);
          Cudd_Ref(tmp4);
          tmp = Cudd_addApply(ddman, Cudd_addTimes, states[i], tmp4); 
          Cudd_Ref(tmp);
          Cudd_RecursiveDeref(ddman, tmp4);
          tmp4 = Cudd_addIte(ddman, tmp, tmp, sol);
          Cudd_Ref(tmp4);
          Cudd_RecursiveDeref(ddman, tmp);
          Cudd_RecursiveDeref(ddman, sol);
          sol = tmp4;
        }
      
			//Cudd_PrintDebug(ddman,sol,2,2);

      // Stop clocks
      stop = util_cpu_time();
      time_for_lp = (double)(stop - start2)/1000;
      time_taken = (double)(stop - start1)/1000;
		} else {
      printf("No solution\n");
    }
    
    printf("\nLP problem solved in %.2f seconds (setup %.2f, lpsolve %.2f) for %d states\n", time_taken, time_for_setup, time_for_lp, nscc);
    
    for (i = 0; i < n; i++) 
      if(states[i]) {
        Cudd_RecursiveDeref(ddman, states[i]);
      }
    // Free memory
    if (ndsm) delete ndsm;
    delete odd;
    delete[] states;
    delete[] map_var;
    delete[] arr_reals;
    delete[] arr_ints;
    delete lp;
    
    Cudd_RecursiveDeref(ddman, tmp3);
    Cudd_RecursiveDeref(ddman, a);
    Cudd_RecursiveDeref(ddman, tmpsol);
    
    // Catch exceptions: register error
  } catch (std::bad_alloc e) {
    printf("Out of memory\n");
    lp_result = NAN;
    if (ndsm) delete ndsm;
    delete odd;
    if(states) delete[] states;
    if(map_var) delete[] map_var;
    if(arr_reals) delete[] arr_reals;
    if(arr_ints) delete[] arr_ints;
    if(lp) delete lp;
    if(lp_soln) delete[] lp_soln;
    Cudd_RecursiveDeref(ddman, tmp3);
    Cudd_RecursiveDeref(ddman, a);
    Cudd_RecursiveDeref(ddman, tmpsol);
  }
}

void processSCC(DdNode *scc, bool trivial) {
  DdNode *a, *tmp, *tmp1, *tmp3, *tmp4;
  DdNode *succ;
  DdNode *onesucc, *onesucc1, *iterscc, *sccone, *sccone1, *tmpsol, *sccsucc, *scctrans01;
  bool prob1;

  /*if(i%1000 == 0) {
    printf("processing scc %d\n", i); 
    fflush(stdout);
    }*/
  // (1) check if this scc needs to be processed
  bool done = false;
  
  DdNode *addscc = Cudd_BddToAdd(ddman, scc);
  Cudd_Ref(addscc);

  if(!Cudd_bddLeq(ddman, scc, bddyes)) { 
    //printf("  computing scc %d\n", i); fflush(stdout);
    // We need to process this scc
    // Get successor states
    scctrans01 = Cudd_bddAnd(ddman, trans03, scc); 
    Cudd_Ref(scctrans01);
    tmp3 = Cudd_bddExistAbstract(ddman, scctrans01, cube);
    Cudd_Ref(tmp3);
    tmp4 = Cudd_bddSwapVariables(ddman, tmp3, rvars, cvars, num_rvars);
    Cudd_Ref(tmp4);
    Cudd_RecursiveDeref(ddman, tmp3);
    sccsucc = Cudd_bddExistAbstract(ddman, tmp4, cube1);
    Cudd_Ref(sccsucc);     
    Cudd_RecursiveDeref(ddman, tmp4);
    succ = Cudd_bddAnd(ddman, sccsucc, Cudd_Not(scc));
    Cudd_Ref(succ);     
    
    prob1 = false;
    if(succ != Cudd_ReadLogicZero(ddman)) {
      //printf("    scc %d has successor states\n", i); fflush(stdout);
      // (2) check if all states have probability one
      prob1 = true;
      //if(zerosucc != Cudd_ReadLogicZero(ddman)) {
      tmp1 = Cudd_addBddThreshold(ddman, sol, 1.0);
      Cudd_Ref(tmp1);
      if(!Cudd_bddLeq(ddman, succ, tmp1)) { 
        //printf("      scc %d has non-one successor states\n", i); fflush(stdout);
        onesucc1 = Cudd_bddOr(ddman, tmp1, scc);
        Cudd_Ref(onesucc1);     
        onesucc = Cudd_bddSwapVariables(ddman, onesucc1, rvars, cvars, num_rvars);
        Cudd_Ref(onesucc);     
        Cudd_RecursiveDeref(ddman, onesucc1);
        tmp3 = Cudd_bddOr(ddman, Cudd_Not(scctrans01), onesucc);
        Cudd_Ref(tmp3);     
        Cudd_RecursiveDeref(ddman, onesucc);
        tmp4 = Cudd_bddUnivAbstract(ddman, tmp3, cube2);
        Cudd_Ref(tmp4);     
        Cudd_RecursiveDeref(ddman, tmp3);
        tmp3 = Cudd_bddAndAbstract(ddman, f, tmp4, cube1);
        Cudd_Ref(tmp3);     
        Cudd_RecursiveDeref(ddman, tmp4);
        
        //if(sbscc != Cudd_ReadLogicZero(ddman)) {
        if(scc != tmp3) {
          prob1 = false;
          Cudd_RecursiveDeref(ddman, tmp3);
        } else if(!trivial) {
          Cudd_RecursiveDeref(ddman, tmp3);
          onesucc1 = Cudd_bddSwapVariables(ddman, tmp1, rvars, cvars, num_rvars);
          Cudd_Ref(onesucc1);
          tmp3 = Cudd_bddOr(ddman, Cudd_Not(scctrans01), onesucc1);
          Cudd_Ref(tmp3);
          Cudd_RecursiveDeref(ddman, onesucc1);
          tmp4 = Cudd_bddUnivAbstract(ddman, tmp3, cube2);
          Cudd_Ref(tmp4);
          Cudd_RecursiveDeref(ddman, tmp3);
          tmp3 = Cudd_bddAndAbstract(ddman, f, tmp4, cube1);
          Cudd_Ref(tmp3);
          Cudd_RecursiveDeref(ddman, tmp4);
          sccone1 = Cudd_bddAnd(ddman, scc, tmp3);
          Cudd_Ref(sccone1);
          Cudd_RecursiveDeref(ddman, tmp3);
          
          if(sccone1 == Cudd_ReadLogicZero(ddman)) 
            prob1 = false;
          
          Cudd_RecursiveDeref(ddman, sccone1);
        }
      }
      
      if(prob1) {
        //printf("        all states in scc %d have probability one\n", i); fflush(stdout);
        tmp3 = Cudd_addApply(ddman, Cudd_addOr, sol, addscc);
        Cudd_Ref(tmp3);
        Cudd_RecursiveDeref(ddman, sol);
        sol = tmp3;
        tmp3 = Cudd_addApply(ddman, Cudd_addOr, probone, addscc);
        Cudd_Ref(tmp3);
        Cudd_RecursiveDeref(ddman, probone);
        probone = tmp3;
      } else {
        //printf(": performing value iteration... "); fflush(stdout);
        // (3) value iteration
        a = Cudd_addApply(ddman, Cudd_addTimes, trans, addscc);
        Cudd_Ref(a);
        tmp3 = Cudd_BddToAdd(ddman, sccsucc);
        Cudd_Ref(tmp3);   
        tmpsol = Cudd_addApply(ddman, Cudd_addTimes, sol, tmp3);
        Cudd_Ref(tmpsol);
        Cudd_RecursiveDeref(ddman, tmp3);
        Cudd_Ref(tmpsol);
        sccone = tmpsol; //Cudd_addApply(ddman, Cudd_addTimes, sol, tmp3);
        
        while(!done) {
          Cudd_Ref(tmpsol);
          iterscc = DD_PermuteVariables(ddman, tmpsol, rvars, cvars, num_rvars);
          Cudd_Ref(a);
          iterscc = DD_MatrixMultiply(ddman, a, iterscc, cvars, num_cvars, MM_BOULDER);
          tmp3 = DD_MaxAbstract(ddman, iterscc, ndvars, num_ndvars);
          
          iterscc = Cudd_addApply(ddman, Cudd_addMaximum, tmp3, sccone);
          Cudd_Ref(iterscc);
          Cudd_RecursiveDeref(ddman, tmp3);
          
          
          if(trivial) {
            Cudd_RecursiveDeref(ddman, tmpsol);
            tmpsol = iterscc;
            break;
          }
          switch (term_crit) {
          case TERM_CRIT_ABSOLUTE:
            if (DD_EqualSupNorm(ddman, iterscc, tmpsol, term_crit_param))
              done = true;
            break;
          case TERM_CRIT_RELATIVE:
            if (DD_EqualSupNormRel(ddman, iterscc, tmpsol, term_crit_param))
              done = true;
            break;
          }
          // prepare for next iteration
          Cudd_RecursiveDeref(ddman, tmpsol);
          tmpsol = iterscc;
        }
        tmp = Cudd_addIte(ddman, tmpsol, tmpsol, sol);
        Cudd_Ref(tmp);
        Cudd_RecursiveDeref(ddman, tmpsol);
        Cudd_RecursiveDeref(ddman, sol);
        sol = tmp;
        /*Cudd_Ref(tmpsol);
          sol = DD_ITE(ddman, tmpsol, tmpsol, sol);*/
        //Cudd_RecursiveDeref(ddman, tmpsol);
        
        Cudd_RecursiveDeref(ddman, a);
        Cudd_RecursiveDeref(ddman, sccone);
      }
      Cudd_RecursiveDeref(ddman, tmp1);
    } else {
      onesucc = Cudd_bddAnd(ddman, bddyes, scc);
      Cudd_Ref(onesucc);
      if(onesucc != Cudd_ReadLogicZero(ddman)) {
        tmp3 = Cudd_addApply(ddman, Cudd_addOr, sol, addscc);
        Cudd_Ref(tmp3);
        Cudd_RecursiveDeref(ddman, sol);
        sol = tmp3;
        tmp3 = Cudd_addApply(ddman, Cudd_addOr, probone, addscc);
        Cudd_Ref(tmp3);
        Cudd_RecursiveDeref(ddman, probone);
        probone = tmp3;
      }
      Cudd_RecursiveDeref(ddman, onesucc);
    }     
    Cudd_RecursiveDeref(ddman, succ);
    Cudd_RecursiveDeref(ddman, sccsucc);
    Cudd_RecursiveDeref(ddman, scctrans01);
  }
  Cudd_RecursiveDeref(ddman, addscc);
}

void processSCC1(DdNode *scc) {
  DdNode *a, *tmp, *tmp3, *tmp4;
  DdNode *iterscc, *sccone, *tmpsol, *sccsucc, *scctrans01;
  long timer1;
  bool done = false;
  
  DdNode *addscc = Cudd_BddToAdd(ddman, scc);
  Cudd_Ref(addscc);

  scctrans01 = Cudd_bddAndAbstract(ddman, trans03, scc, cube); 
  Cudd_Ref(scctrans01);
  tmp4 = Cudd_bddSwapVariables(ddman, scctrans01, rvars, cvars, num_rvars);
  Cudd_Ref(tmp4);
  Cudd_RecursiveDeref(ddman, scctrans01);
  sccsucc = Cudd_bddExistAbstract(ddman, tmp4, cube1);
  Cudd_Ref(sccsucc);     
  Cudd_RecursiveDeref(ddman, tmp4);
  
  //printf(": performing value iteration...\n "); fflush(stdout);
  a = Cudd_addApply(ddman, Cudd_addTimes, trans, addscc);
  Cudd_Ref(a);
  tmp3 = Cudd_BddToAdd(ddman, sccsucc);
  Cudd_Ref(tmp3);   
  Cudd_RecursiveDeref(ddman, sccsucc);
  tmpsol = Cudd_addApply(ddman, Cudd_addTimes, sol, tmp3);
  Cudd_Ref(tmpsol);
  Cudd_RecursiveDeref(ddman, tmp3);
  Cudd_Ref(tmpsol);
  sccone = tmpsol; 
  
  while(!done) {
    iters++;
    iterscc = Cudd_addSwapVariables(ddman, tmpsol, rvars, cvars, num_rvars);
    Cudd_Ref(iterscc);
    timer1 = util_cpu_time();
    tmp4 = Cudd_addTimesPlus/*Cudd_addMatrixMultiply*/(ddman, a, iterscc, cvars, num_cvars);
    Cudd_Ref(tmp4);
    count7 += util_cpu_time() - timer1;
    Cudd_RecursiveDeref(ddman, iterscc);
    if (Pmin) {
      sccone = Cudd_addApply(ddman, Cudd_addTimes, mask, addscc);
      Cudd_Ref(sccone);
      tmp = Cudd_addApply(ddman, Cudd_addMaximum, tmp4, sccone);
      Cudd_Ref(tmp);
      Cudd_RecursiveDeref(ddman, sccone);
      tmp3 = Cudd_addMinAbstract(ddman, tmp, addcube1);
      Cudd_Ref(tmp3);
      Cudd_RecursiveDeref(ddman, tmp);
    } else {
      tmp3 = Cudd_addMaxAbstract(ddman, tmp4, addcube1);
      Cudd_Ref(tmp3);
    }
    Cudd_RecursiveDeref(ddman, tmp4);

    iterscc = Cudd_addApply(ddman, Cudd_addMaximum, tmp3, sccone);
    Cudd_Ref(iterscc);
    Cudd_RecursiveDeref(ddman, tmp3);
    
    switch (term_crit) {
    case TERM_CRIT_ABSOLUTE:
      if (DD_EqualSupNorm(ddman, iterscc, tmpsol, term_crit_param)) {
        done = true;
      }
      break;
    case TERM_CRIT_RELATIVE:
      if (DD_EqualSupNormRel(ddman, iterscc, tmpsol, term_crit_param)) {
        done = true;
      }
      break;
    }
    // prepare for next iteration
    Cudd_RecursiveDeref(ddman, tmpsol);
    tmpsol = iterscc;
    //printf("      tmpsol:\n"); fflush(stdout);
    //Cudd_PrintDebug(ddman, tmpsol, num_rvars, 2);
  }
  tmp = Cudd_addIte(ddman, tmpsol, tmpsol, sol);
  Cudd_Ref(tmp);
  Cudd_RecursiveDeref(ddman, tmpsol);
  Cudd_RecursiveDeref(ddman, sol);
  sol = tmp;
  
  Cudd_RecursiveDeref(ddman, a);
  Cudd_RecursiveDeref(ddman, sccone);
  Cudd_RecursiveDeref(ddman, addscc);
}

void processSCC2(DdNode *scc) {
  DdNode *a, *bdda, *tmp, *tmp1, *tmp3, *tmp4;
  DdNode *iterscc, *tmpsol, *tmpa, *sccsucc, *scctrans01;
  long timer1;
  bool done = false;
  
  DdNode *addscc = Cudd_BddToAdd(ddman, scc);
  Cudd_Ref(addscc);

  scctrans01 = Cudd_bddAndAbstract(ddman, trans03, scc, cube); 
  Cudd_Ref(scctrans01);
  tmp4 = Cudd_bddSwapVariables(ddman, scctrans01, rvars, cvars, num_rvars);
  Cudd_Ref(tmp4);
  Cudd_RecursiveDeref(ddman, scctrans01);
  sccsucc = Cudd_bddExistAbstract(ddman, tmp4, cube1);
  Cudd_Ref(sccsucc);     
  Cudd_RecursiveDeref(ddman, tmp4);
  
  //printf(": performing value iteration...\n "); fflush(stdout);
  a = Cudd_addApply(ddman, Cudd_addTimes, trans, addscc);
  Cudd_Ref(a);
  tmp3 = Cudd_BddToAdd(ddman, sccsucc);
  Cudd_Ref(tmp3);   
  Cudd_RecursiveDeref(ddman, sccsucc);
  tmpsol = Cudd_addApply(ddman, Cudd_addTimes, sol, tmp3);
  Cudd_Ref(tmpsol);
  Cudd_RecursiveDeref(ddman, tmp3);
  //Cudd_Ref(tmpsol);
  //sccone = tmpsol; 
  
  iters++;
  iterscc = Cudd_addSwapVariables(ddman, tmpsol, rvars, cvars, num_rvars);
  Cudd_Ref(iterscc);
  timer1 = util_cpu_time();
  tmp4 = Cudd_addTimesPlus/*Cudd_addMatrixMultiply*/(ddman, a, iterscc, cvars, num_cvars);
  Cudd_Ref(tmp4);
  count7 += util_cpu_time() - timer1;
  Cudd_RecursiveDeref(ddman, iterscc);
  tmp3 = Cudd_addMaxAbstract(ddman, tmp4, addcube1);
  Cudd_Ref(tmp3);
  Cudd_RecursiveDeref(ddman, tmp4);
  
  // get states that have new probability value
  // start with ADD
  timer1 = util_cpu_time();
  bdda = Cudd_bddAnd(ddman, trans04, scc);
  Cudd_Ref(bdda);
  tmp4 = Cudd_addApply(ddman, Cudd_addMinus, tmp3, tmpsol);
  Cudd_Ref(tmp4);
  tmp1 = Cudd_addBddStrictThreshold(ddman, tmp4, 0.0);
  Cudd_Ref(tmp1);
  Cudd_RecursiveDeref(ddman, tmp4);
  // Now we use BDD to get the states that have new value
  tmp4 = Cudd_bddSwapVariables(ddman, tmp1, bddrvars, bddcvars, num_rvars);
  Cudd_Ref(tmp4);
  Cudd_RecursiveDeref(ddman, tmp1);
  tmp1 = Cudd_bddAndAbstract(ddman, bdda, tmp4, cube2);
  Cudd_Ref(tmp1);
  Cudd_RecursiveDeref(ddman, tmp4);
  if(tmp1 == Cudd_ReadLogicZero(ddman)) {
    //printf("    Found a trivial scc\n "); fflush(stdout);    
    done = true; // the scc is trivial (single state without selfloop)
    Cudd_RecursiveDeref(ddman, tmp1);
  } else {
    // Now change back to ADD
    tmp4 = Cudd_BddToAdd(ddman, tmp1);
    Cudd_Ref(tmp4);
    Cudd_RecursiveDeref(ddman, tmp1);
    // Get transition relation for those states
    tmpa = Cudd_addApply(ddman, Cudd_addTimes, a, tmp4);
    Cudd_Ref(tmpa);
    Cudd_RecursiveDeref(ddman, tmp4);
  }
  count6 += util_cpu_time() - timer1;

  iterscc = Cudd_addApply(ddman, Cudd_addMaximum, tmp3, tmpsol);
  Cudd_Ref(iterscc);
  Cudd_RecursiveDeref(ddman, tmp3);
  // prepare for next iteration
  Cudd_RecursiveDeref(ddman, tmpsol);
  tmpsol = iterscc;
  //printf("      tmpsol:\n"); fflush(stdout);
  //Cudd_PrintDebug(ddman, tmpsol, num_rvars, 2);

  while(!done) {
    iters++;
    iterscc = Cudd_addSwapVariables(ddman, tmpsol, rvars, cvars, num_rvars);
    Cudd_Ref(iterscc);
    timer1 = util_cpu_time();
    tmp4 = Cudd_addTimesPlus/*Cudd_addMatrixMultiply*/(ddman, tmpa, iterscc, cvars, num_cvars);
    Cudd_Ref(tmp4);
    Cudd_RecursiveDeref(ddman, tmpa);
    count7 += util_cpu_time() - timer1;
    Cudd_RecursiveDeref(ddman, iterscc);
    tmp3 = Cudd_addMaxAbstract(ddman, tmp4, addcube1);
    Cudd_Ref(tmp3);
    Cudd_RecursiveDeref(ddman, tmp4);

    // get states that have new probability value
    // start with ADD
    timer1 = util_cpu_time();
    tmp4 = Cudd_addApply(ddman, Cudd_addMinus, tmp3, tmpsol);
    Cudd_Ref(tmp4);
    tmp1 = Cudd_addBddStrictThreshold(ddman, tmp4, 0.0);
    Cudd_Ref(tmp1);
    Cudd_RecursiveDeref(ddman, tmp4);
    // Now we use BDD to get the states that have new value
    tmp4 = Cudd_bddSwapVariables(ddman, tmp1, bddrvars, bddcvars, num_rvars);
    Cudd_Ref(tmp4);
    Cudd_RecursiveDeref(ddman, tmp1);
    tmp1 = Cudd_bddAndAbstract(ddman, bdda, tmp4, cube2);
    Cudd_Ref(tmp1);
    Cudd_RecursiveDeref(ddman, tmp4);
    // Now change back to ADD
    tmp4 = Cudd_BddToAdd(ddman, tmp1);
    Cudd_Ref(tmp4);
    Cudd_RecursiveDeref(ddman, tmp1);
    // Get transition relation for those states
    tmpa = Cudd_addApply(ddman, Cudd_addTimes, a, tmp4);
    Cudd_Ref(tmpa);
    Cudd_RecursiveDeref(ddman, tmp4);
    count6 += util_cpu_time() - timer1;

    iterscc = Cudd_addApply(ddman, Cudd_addMaximum, tmp3, tmpsol);
    Cudd_Ref(iterscc);
    Cudd_RecursiveDeref(ddman, tmp3);
    
    switch (term_crit) {
    case TERM_CRIT_ABSOLUTE:
      if (DD_EqualSupNorm(ddman, iterscc, tmpsol, term_crit_param)) {
        done = true;
        Cudd_RecursiveDeref(ddman, tmpa);
      }
      break;
    case TERM_CRIT_RELATIVE:
      if (DD_EqualSupNormRel(ddman, iterscc, tmpsol, term_crit_param)) {
        done = true;
        Cudd_RecursiveDeref(ddman, tmpa);
      }
      break;
    }
    // prepare for next iteration
    Cudd_RecursiveDeref(ddman, tmpsol);
    tmpsol = iterscc;
    //printf("      tmpsol:\n"); fflush(stdout);
    //Cudd_PrintDebug(ddman, tmpsol, num_rvars, 2);
  }
  tmp = Cudd_addIte(ddman, tmpsol, tmpsol, sol);
  Cudd_Ref(tmp);
  Cudd_RecursiveDeref(ddman, tmpsol);
  Cudd_RecursiveDeref(ddman, sol);
  sol = tmp;
  
  Cudd_RecursiveDeref(ddman, a);
  Cudd_RecursiveDeref(ddman, bdda);
  //Cudd_RecursiveDeref(ddman, sccone);
  Cudd_RecursiveDeref(ddman, addscc);
}

void processSCC3(DdNode *scc) {
  DdNode *a, *tmp, *tmp3, *tmp4;
  DdNode *iterscc, *sccone, *tmpsol, *sccsucc, *scctrans01;
  long timer1;
  //bool done = false;
  
  DdNode *addscc = Cudd_BddToAdd(ddman, scc);
  Cudd_Ref(addscc);

  scctrans01 = Cudd_bddAndAbstract(ddman, trans03, scc, cube); 
  Cudd_Ref(scctrans01);
  tmp4 = Cudd_bddSwapVariables(ddman, scctrans01, rvars, cvars, num_rvars);
  Cudd_Ref(tmp4);
  Cudd_RecursiveDeref(ddman, scctrans01);
  sccsucc = Cudd_bddExistAbstract(ddman, tmp4, cube1);
  Cudd_Ref(sccsucc);     
  Cudd_RecursiveDeref(ddman, tmp4);
  
  //printf(": performing value iteration...\n "); fflush(stdout);
  a = Cudd_addApply(ddman, Cudd_addTimes, trans, addscc);
  Cudd_Ref(a);
  tmp3 = Cudd_BddToAdd(ddman, sccsucc);
  Cudd_Ref(tmp3);   
  Cudd_RecursiveDeref(ddman, sccsucc);
  tmpsol = Cudd_addApply(ddman, Cudd_addTimes, sol, tmp3);
  Cudd_Ref(tmpsol);
  Cudd_RecursiveDeref(ddman, tmp3);
  //Cudd_Ref(tmpsol);
  //sccone = tmpsol; 
  
  iters++;
  iterscc = Cudd_addSwapVariables(ddman, tmpsol, rvars, cvars, num_rvars);
  Cudd_Ref(iterscc);
  timer1 = util_cpu_time();
  tmp4 = Cudd_addTimesPlus/*Cudd_addMatrixMultiply*/(ddman, a, iterscc, cvars, num_cvars);
  Cudd_Ref(tmp4);
  count7 += util_cpu_time() - timer1;
  Cudd_RecursiveDeref(ddman, iterscc);
  if (Pmin) {
    sccone = Cudd_addApply(ddman, Cudd_addTimes, mask, addscc);
    Cudd_Ref(sccone);
    tmp = Cudd_addApply(ddman, Cudd_addMaximum, tmp4, sccone);
    Cudd_Ref(tmp);
    Cudd_RecursiveDeref(ddman, sccone);
    tmp3 = Cudd_addMinAbstract(ddman, tmp, addcube1);
    Cudd_Ref(tmp3);
    Cudd_RecursiveDeref(ddman, tmp);
  } else {
    tmp3 = Cudd_addMaxAbstract(ddman, tmp4, addcube1);
    Cudd_Ref(tmp3);
  }
  Cudd_RecursiveDeref(ddman, tmp4);
  
  //iterscc = Cudd_addApply(ddman, Cudd_addMaximum, tmp3, sccone);
  //Cudd_Ref(iterscc);
  //Cudd_RecursiveDeref(ddman, tmp3);

  // prepare for next iteration
  Cudd_RecursiveDeref(ddman, tmpsol);
  //tmpsol = iterscc;
  tmpsol = tmp3;

  //printf("      tmpsol:\n"); fflush(stdout);
  //Cudd_PrintDebug(ddman, tmpsol, num_rvars, 2);
  tmp = Cudd_addIte(ddman, tmpsol, tmpsol, sol);
  Cudd_Ref(tmp);
  Cudd_RecursiveDeref(ddman, tmpsol);
  Cudd_RecursiveDeref(ddman, sol);
  sol = tmp;
  
  Cudd_RecursiveDeref(ddman, a);
  //Cudd_RecursiveDeref(ddman, sccone);
  Cudd_RecursiveDeref(ddman, addscc);
}

result visit2(DdNode *v, int index) {
  DdNode *w, *w0, *tmp;
  map<DdNode *, int>::iterator it;
  int n;

  if((index-1) % dividend == 0)
    printf("exploring %d state ...\n", index-1);
  //long timer1 = util_cpu_time();
  int index1 = index+1;
  int y = index;
  (*S2)[v] = index;
  //count4 += util_cpu_time() - timer1;
  
  //timer1 = util_cpu_time();
  DdNode *successors = Cudd_bddAndAbstract(ddman, trans02, v, cube);
  Cudd_Ref(successors);
  w0 = Cudd_bddSwapVariables(ddman, successors, bddrvars, bddcvars, num_rvars);
  Cudd_Ref(w0);
  Cudd_RecursiveDeref(ddman, successors);
  successors = w0;
  //count1 += util_cpu_time() - timer1;
  
  struct result r;
  while(successors != Cudd_ReadLogicZero(ddman)) {
    int x = 1;
    //timer1 = util_cpu_time();
    w = Cudd_bddPickOneMinterm(ddman, successors, bddrvars, num_rvars);
    Cudd_Ref(w);
    //count2 += util_cpu_time() - timer1;
    it = S2->find(w);
    if(it == S2->end()) {
      if(!Cudd_bddLeq(ddman, w, allsccs)) {
        r = visit2(w, index1);
        index1 = r.index;
        x = r.w0;
      } 
    } else {
      x = it->second;
    }
    
    if(x > 1 && y > x) {
      ///timer1 = util_cpu_time();
      (*S2)[v] = x;
      ///count4 += util_cpu_time() - timer1;
      y = x;
    }
    
    //timer1 = util_cpu_time();
    w0 = Cudd_bddAnd(ddman, successors, Cudd_Not(w));
    Cudd_Ref(w0);
    Cudd_RecursiveDeref(ddman, w);
    Cudd_RecursiveDeref(ddman, successors);
    successors = w0;
    //count2 += util_cpu_time() - timer1;
  }
  Cudd_RecursiveDeref(ddman, successors);
  
  if(y == index) {
    y = 1;
    //timer1 = util_cpu_time();
    S2->erase(S2->find(v));
    //count4 += util_cpu_time() - timer1;
    
    DdNode *scc = v;
    n = 1;
    while(!S->empty()) {
      tmp = S->back();
      it = S2->find(tmp);
      if(it->second >= index) {
        //timer1 = util_cpu_time();
        S2->erase(it);
        //count4 += util_cpu_time() - timer1;
        S->pop_back();
        //timer1 = util_cpu_time();
        w0 = Cudd_bddOr(ddman, scc, tmp);
        Cudd_Ref(w0);
        Cudd_RecursiveDeref(ddman, tmp);
        if(n > 1)
          Cudd_RecursiveDeref(ddman, scc);
        n++;
        scc = w0;
        //count5 += util_cpu_time() - timer1;
      } else 
        break;
    }
    if(n==1)
      Cudd_Ref(scc);
    //timer1 = util_cpu_time();
    w0 = Cudd_bddOr(ddman, allsccs, scc);
    Cudd_Ref(w0);
    Cudd_RecursiveDeref(ddman, allsccs);
    allsccs = w0;
    //count4 += util_cpu_time() - timer1;
    
    // perform value iteration on scc
    long timer1 = util_cpu_time();
    if(n < LPBOUND) {
      if(n==1 && !Cudd_bddLeq(ddman, scc, nontrivialsccs)) // scc is a trivial scc
        processSCC3(scc);
      else
        processSCC1(scc);
    } else
      processSCC_LP(scc, n);
    count1 += util_cpu_time() - timer1;
    Cudd_RecursiveDeref(ddman, scc);
    num_sccs++;
  } else {
    Cudd_Ref(v);
    S->push_back(v);
  }
  struct result res={index1, y};
  return res;
}

//------------------------------------------------------------------------------

JNIEXPORT jlong __jlongpointer JNICALL Java_mtbdd_PrismMTBDD_PM_1SCCBasedNondetUntil
(
 JNIEnv *env,
 jclass cls,
 jlong __jlongpointer t,  // trans matrix
 jlong __jlongpointer t01, // trans01
 jlong __jlongpointer od, // odd
 jlong __jlongpointer ndm,  // nondeterminism mask
 jlong __jlongpointer init, // initial state
 jlong __jlongpointer rv, // row vars
 jint num_rvars1,
 jlong __jlongpointer cv, // col vars
 jint num_cvars1,
 jlong __jlongpointer ndv,  // nondet vars
 jint num_ndvars1,
 jlong __jlongpointer y,  // 'yes' states
 jlong __jlongpointer m,  // 'maybe' states
 jboolean min,    // min or max probabilities (true = min, false = max)
 jboolean opt   
 //jlongArray _sccs // SCCs
 )
{
  // cast function parameters
  /*DdNode *trans = jlong_to_DdNode(t);   // trans matrix
    DdNode *trans01 = jlong_to_DdNode(t01);       // trans01 matrix
    ODDNode *odd = jlong_to_ODDNode(od);    // odd
    DdNode *mask = jlong_to_DdNode(ndm);    // nondeterminism mask
    DdNode *start = jlong_to_DdNode(init);       // initial state
    DdNode **rvars = jlong_to_DdNode_array(rv); // row vars
    DdNode **cvars = jlong_to_DdNode_array(cv); // col vars
    DdNode **ndvars = jlong_to_DdNode_array(ndv); // nondet vars
    DdNode *yes = jlong_to_DdNode(y);   // 'yes' states
    DdNode *maybe = jlong_to_DdNode(m);   // 'maybe' states*/

  num_rvars = num_rvars1;
  num_cvars = num_cvars1;
  num_ndvars = num_ndvars1;

  // mtbdds
  DdNode *tmp, *tmp1, *tmp2, *tmp3, *tmp4, /* *trans02,*/ *selfloop1;
  // timing stuff
  long start1, start2, stop;
  double time_taken, time_for_setup, time_for_iters;
  // misc
  bool done;
  int i;

  trans = jlong_to_DdNode(t);   // trans matrix
  DdNode *trans01 = jlong_to_DdNode(t01);       // trans01 matrix
  mask = jlong_to_DdNode(ndm);    // nondeterminism mask
  DdNode *start = jlong_to_DdNode(init);       // initial state
  rvars = jlong_to_DdNode_array(rv);  // row vars
  cvars = jlong_to_DdNode_array(cv);  // col vars
  ndvars = jlong_to_DdNode_array(ndv);  // nondet vars
  DdNode *yes = jlong_to_DdNode(y);   // 'yes' states
  DdNode *maybe = jlong_to_DdNode(m);   // 'maybe' states
  Pmin = min;

  // start clocks
  start1 = start2 = util_cpu_time();
  
  // initial solution
  // (prob in 0 steps given by yes)
  Cudd_Ref(yes);
  sol = yes;
  Cudd_Ref(yes);
  probone = yes;
  bddyes = Cudd_addBddThreshold(ddman, yes, 1.0);
  Cudd_Ref(bddyes);  
  //printf("initial sol:\n");
  //Cudd_PrintDebug(ddman, sol, num_rvars, 2);
  /*
   * Compute SCC using Tarjan's algorithm
   */
  S = new vector<DdNode *>();
  //vector<int> S1;
  //vector<DdNode *> sccs1;
  DdNode *bddstart;
  bddrvars = new DdNode *[num_rvars];
  bddcvars = new DdNode *[num_cvars];
  bddndvars = new DdNode *[num_ndvars];
  //DdNode *nodeinfo0 = Cudd_ReadZero(ddman);
  //Cudd_Ref(nodeinfo0);
  //remove selfloop
  DdNode *selfloop = Cudd_ReadOne(ddman);
  Cudd_Ref(selfloop);
  for(i=0; i<num_rvars; i++) {
    tmp1 = Cudd_bddIthVar(ddman, rvars[i]->index);
    tmp2 = Cudd_bddIthVar(ddman, cvars[i]->index);
    tmp3 = Cudd_bddAnd(ddman, tmp1, tmp2);
    Cudd_Ref(tmp3);
    tmp4 = Cudd_bddAnd(ddman, Cudd_Not(tmp1), Cudd_Not(tmp2));
    Cudd_Ref(tmp4);
    tmp = Cudd_bddAnd(ddman, selfloop, Cudd_bddOr(ddman, tmp3, tmp4));
    Cudd_Ref(tmp);
    Cudd_RecursiveDeref(ddman, tmp3);
    Cudd_RecursiveDeref(ddman, tmp4);
    Cudd_RecursiveDeref(ddman, selfloop);
    selfloop = tmp;

    bddrvars[i] = Cudd_bddIthVar(ddman, rvars[i]->index);
    bddcvars[i] = Cudd_bddIthVar(ddman, cvars[i]->index);
  }
  selfloop1 = Cudd_BddToAdd(ddman, selfloop);
  Cudd_Ref(selfloop1);  
  cube = Cudd_bddComputeCube(ddman, bddrvars, NULL, num_rvars);
  Cudd_Ref(cube);  
  cube2 = Cudd_bddComputeCube(ddman, bddcvars, NULL, num_cvars);
  Cudd_Ref(cube2);  
  for(i=0; i<num_ndvars; i++) 
    bddndvars[i] = Cudd_bddIthVar(ddman, ndvars[i]->index);
  cube1 = Cudd_bddComputeCube(ddman, bddndvars, NULL, num_ndvars);
  Cudd_Ref(cube1);  
  tmp = Cudd_addBddThreshold(ddman, trans01, 1);
  Cudd_Ref(tmp);
  
  tmp1 = Cudd_bddAndAbstract(ddman, tmp, selfloop, cube2);
  Cudd_Ref(tmp1);
  nontrivialsccs = Cudd_bddExistAbstract(ddman, tmp1, cube1);
  Cudd_Ref(nontrivialsccs);
  Cudd_RecursiveDeref(ddman, tmp1);
  
  f = Cudd_bddUnivAbstract(ddman, tmp, cube2);
  Cudd_Ref(f); 
  trans03 = Cudd_bddAnd(ddman, tmp, Cudd_Not(selfloop));
  Cudd_Ref(trans03);
  Cudd_RecursiveDeref(ddman, tmp);
  //trans03 = tmp;

  trans02 = Cudd_bddExistAbstract(ddman, trans03, cube1);
  Cudd_Ref(trans02);
  //Cudd_RecursiveDeref(ddman, cube1);
  Cudd_RecursiveDeref(ddman, selfloop);
  
  
  // Remove yes and no states
  tmp = Cudd_addBddThreshold(ddman, maybe, 1);
  Cudd_Ref(tmp);
  dividend = (int)(Cudd_CountMinterm(ddman, tmp, num_rvars)/1000.0);
  if(dividend <1000)
    dividend = 1000;
  else if (dividend <10000)
    dividend = 10000;
  else 
    dividend = 100000;

  tmp1 = Cudd_bddAnd(ddman, nontrivialsccs, tmp);
  Cudd_Ref(tmp1);
  Cudd_RecursiveDeref(ddman, nontrivialsccs);
  nontrivialsccs = tmp1;

  tmp1 = Cudd_bddAnd(ddman, trans02, tmp);
  Cudd_Ref(tmp1);
  tmp3 = Cudd_bddSwapVariables(ddman, tmp, rvars, cvars, num_rvars);
  Cudd_Ref(tmp3);
  Cudd_RecursiveDeref(ddman, tmp);
  tmp = Cudd_bddAnd(ddman, tmp1, tmp3);
  Cudd_Ref(tmp);
  Cudd_RecursiveDeref(ddman, tmp1);
  Cudd_RecursiveDeref(ddman, tmp3);
  Cudd_RecursiveDeref(ddman, trans02);  
  trans02 = tmp;


  // Recursive call
  bddstart = Cudd_addBddThreshold(ddman, start, 1);
  Cudd_Ref(bddstart);
  S2 = new map<DdNode *, int>();
  allsccs = Cudd_ReadLogicZero(ddman);
  Cudd_Ref(allsccs);

  // remove selfloop from trans
  tmp1 = Cudd_addApply(ddman, Cudd_addTimes, trans, maybe);
  Cudd_Ref(tmp1);
  tmp = Cudd_addApply(ddman, Cudd_addTimes, tmp1, selfloop1);
  Cudd_Ref(tmp);
  Cudd_RecursiveDeref(ddman, tmp1);
  Cudd_RecursiveDeref(ddman, selfloop1);
  tmp1 = Cudd_addBddThreshold(ddman, tmp, 1.0);
  Cudd_Ref(tmp1);
  Cudd_RecursiveDeref(ddman, tmp);
  selfloop1 = Cudd_BddToAdd(ddman, tmp1);
  Cudd_Ref(selfloop1);
  Cudd_RecursiveDeref(ddman, tmp1);
  tmp1 = Cudd_addCmpl(ddman, selfloop1);
  Cudd_Ref(tmp1);
  Cudd_RecursiveDeref(ddman, selfloop1);
  tmp = Cudd_addApply(ddman, Cudd_addTimes, trans, tmp1);
  Cudd_Ref(tmp);
  Cudd_RecursiveDeref(ddman, tmp1);
  trans = tmp;
  // convert it into bdd
  tmp1 = Cudd_addBddStrictThreshold(ddman, trans, 0.0);
  Cudd_Ref(tmp1);
  trans04 = Cudd_bddExistAbstract(ddman, tmp1, cube1);
  Cudd_Ref(trans04);
  Cudd_RecursiveDeref(ddman, tmp1);

  // Compute add cube
  addcube1 = Cudd_addComputeCube(ddman, ndvars, NULL, num_ndvars);
  Cudd_Ref(addcube1);  
  //addcube2 = Cudd_addComputeCube(ddman, cvars, NULL, num_cvars);
  //Cudd_Ref(addcube2);  


  // get setup time
  stop = util_cpu_time();
  time_for_setup = (double)(stop - start2)/1000;
  start2 = stop;

  printf("Computing SCCs...\n"); fflush(stdout);
  long timer1 = util_cpu_time();
  /*visit1(ddman, bddstart, 2, &nodeinfo0, cube,
    &trans02, bddrvars, bddcvars, num_rvars, num_cvars, num_ndvars,
    &count1, &count2, &count3, &count4, &count5, &count6, &count7,
    &S, &S1, &sccs1);*/
  visit2(bddstart, 2);
  timer1 = util_cpu_time() - timer1;
  printf("done\nfind %d sccs in %fs (solving LP in %fs)\n", /*sccs1.size()*/num_sccs, (timer1/1000.0), (count1/1000.0)); fflush(stdout);
  Cudd_RecursiveDeref(ddman, bddstart);
  //Cudd_RecursiveDeref(ddman, nodeinfo0);
  Cudd_RecursiveDeref(ddman, trans02);
  Cudd_RecursiveDeref(ddman, allsccs);
  //for(i=0; i<sccs1.size(); i++) 
  //  Cudd_RecursiveDeref(ddman, sccs1[i]);
  //sccs1.clear();
  //printf("time to extract successor states = %fs\n", (count1/1000.0));
  //printf("time to get a successor state = %fs\n", (count2/1000.0));
  //printf("time to check successor state status= %fs\n", (count3/1000.0));
  //printf("time to update nodeinfo0 = %fs\n", (count4/1000.0));
  //printf("time to join scc = %fs\n", (count5/1000.0));
  //printf("time to get updated states = %fs\n", (count6/1000.0));
  //printf("time to multiply matrix = %fs\n\n", (count7/1000.0));

  Cudd_RecursiveDeref(ddman, probone);
  Cudd_RecursiveDeref(ddman, f);
  done = true;

  
  /*for(i=0; i<num_sccs; i++) {
  //Cudd_RecursiveDeref(ddman, sccs[i]);
  Cudd_RecursiveDeref(ddman, sccs1[i]);
  }
  sccs1.clear();*/
  delete S;
  delete S2;
  delete[] bddrvars;
  delete[] bddcvars;
  delete[] bddndvars;
  Cudd_RecursiveDeref(ddman, trans);
  Cudd_RecursiveDeref(ddman, trans04);
  Cudd_RecursiveDeref(ddman, addcube1);
  //Cudd_RecursiveDeref(ddman, addcube2);
  Cudd_RecursiveDeref(ddman, bddyes);
  Cudd_RecursiveDeref(ddman, trans03);
  Cudd_RecursiveDeref(ddman, cube2);
  Cudd_RecursiveDeref(ddman, cube1);
  Cudd_RecursiveDeref(ddman, cube);
  Cudd_RecursiveDeref(ddman, nontrivialsccs);

  // stop clocks
  stop = util_cpu_time();
  time_for_iters = (double)(stop - start2)/1000;
  time_taken = (double)(stop - start1)/1000;
  
  // print iterations/timing info
  PM_PrintToMainLog(env, "\nIterative method: %d iterations in %.2f seconds (average %.6f, setup %.2f)\n", 
                    iters, time_taken, time_for_iters/iters, time_for_setup);
  
  // if the iterative method didn't terminate, this is an error
  if (!done) { 
    Cudd_RecursiveDeref(ddman, sol); 
    PM_SetErrorMessage("Iterative method did not converge within %d iterations.\nConsider using a different numerical method or increasing the maximum number of iterations", iters); return 0; 
  }
  
  return ptr_to_jlong(sol);
}

//------------------------------------------------------------------------------
